@@ -27,10 +27,13 @@ from pyngrok import conf, ngrok
 @click.option('--build','-b',help="True or False",default=True)
 @click.option('--api-domain','-api-domain',help="API domain for Core Services(common.appfire.app or dts.common.appfire.app or services.custom-domain)")
 @click.option('--api-stage','-api-stage',help="API stage for Core Services(prod or snapshot or stage)")
+@click.option('--app-api-domain','-app-api-domain',help="API domain for App Services")
 @click.option('--require-approval','-require-approval',help="never or any-change or broadening",default="never")
+@click.option('--tag','-t',help="service module image tag",default="latest")
+@click.option('--dts-ecr-profile', '-dts-ecr-profile', help="AWS profile for DTS ECR")
 @click.argument('command')
 @click.help_option("--help", "-h")
-def process(verbose, command, profile, env, stage, stack, app_suffix, build, api_domain, api_stage, require_approval):
+def process(verbose, command, profile, env, stage, stack, app_suffix, build, api_domain, api_stage, app_api_domain, require_approval, tag, dts_ecr_profile):
     """
     Gathers information related to deployment and deploys the CDK stack
 
@@ -42,7 +45,7 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
       build:dev     Build the app with webpack-dev.config.js config
         Options:  --profile, --env, --stage, --stack, --api-domain, --api-stage
       deploy        Build and deploy the specified stack
-        Options:  --profile, --stack
+        Options:  --profile, --stack, --tag
       diff          Diff the local stack with deployed stack
         Options:  --profile
       synth         Generates AWS CloudFormation template from the app
@@ -52,7 +55,7 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
       list          List the stacks for the app
         Options:  --profile
       run           Build and run the app with webpack-dev.config.js config
-        Options:  --profile, --env, --stage, --stack, --api-domain, --api-stage
+        Options:  --profile, --env, --stage, --stack, --api-domain, --api-stage, --app-api-domain
       version       Print version information for the Appfire SDK
 
     Learn more - https://wiki.appfire.com/x/x6gjCQ
@@ -68,13 +71,16 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
     if command.lower() == 'version':
         shell(logger,"pip show appfire-connect-sdk",raise_error=True)
         return
+    if command.lower() == 'dts-ecr-login':
+        dts_ecr_login(logger,dts_ecr_profile)
+        return
     
     logger.debug("checking environment " + env)
     if env == 'standalone':
         personal_env_settings = None
         try:
             with open("./personal.env.yml", 'r') as stream:
-                    personal_env_settings = yaml.safe_load(stream)
+                personal_env_settings = yaml.safe_load(stream)
         except yaml.YAMLError as exc:
             logger.info(exc)
         except IOError:
@@ -96,10 +102,18 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         else :
             logger.info("CDK_DEPLOY_ACCOUNT and CDK_DEPLOY_REGION is not found in personal.env.yml.")
         #### Set API_DOMAIN, API_STAGE, API_PATH as env variable
+       
+        if app_api_domain :
+            app_api_domain = app_api_domain
+        elif 'app_api_domain' in personal_env_settings['environment']['personal'].keys() :
+            app_api_domain =  personal_env_settings['environment']['personal']['app_api_domain']
+        else :
+            app_api_domain = ""
         api_domain = api_domain if api_domain else personal_env_settings['environment']['personal']['api_domain']
         api_stage =  api_stage if api_stage else personal_env_settings['environment']['personal']['api_stage']
         api_path = api_domain+"/"+api_stage
-        logger.info("Setting up Env variables : API_DOMAIN {}, API_STAGE {}, API_PATH {}".format(api_domain, api_stage, api_path))
+        logger.info("Setting up Env variables : APP_API_DOMAIN {}, API_DOMAIN {}, API_STAGE {}, API_PATH {}".format(app_api_domain, api_domain, api_stage, api_path))
+        os.environ["APP_API_DOMAIN"] = app_api_domain
         os.environ["API_DOMAIN"] = api_domain
         os.environ["API_STAGE"] = api_stage
         os.environ["API_PATH"] = api_path
@@ -115,7 +129,16 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         aws_profile = profile if profile else dts_env_settings['environment'][env][stage]['profile']
         domain = dts_env_settings['environment'][env][stage]['domain']
         log_info(logger, aws_profile, domain, env)
-        #### Set API_DOMAIN, API_STAGE, API_PATH as env variable
+        #### Set APP_API_DOMAIN, API_DOMAIN, API_STAGE, API_PATH as env variable
+        if app_api_domain :
+            app_api_domain = app_api_domain
+        elif 'app_api_domain' in dts_env_settings['environment'][env][stage].keys() :
+            app_api_domain =  dts_env_settings['environment'][env][stage]['app_api_domain']
+        elif 'app_api_domain' in dts_env_settings['environment'][env]["default"].keys() :
+            app_api_domain =  dts_env_settings['environment'][env]["default"]['app_api_domain']
+        else :
+            app_api_domain = ""
+
         if api_domain :
             api_domain = api_domain
         elif 'api_domain' in dts_env_settings['environment'][env][stage].keys() :
@@ -135,7 +158,8 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
             api_stage = api_stage
 
         api_path = api_domain+"/"+api_stage
-        logger.info("Setting up Env variables : API_DOMAIN {}, API_STAGE {}, API_PATH {}".format(api_domain, api_stage, api_path))
+        logger.info("Setting up Env variables : APP_API_DOMAIN {}, API_DOMAIN {}, API_STAGE {}, API_PATH {}".format(app_api_domain, api_domain, api_stage, api_path))
+        os.environ["APP_API_DOMAIN"] = app_api_domain
         os.environ["API_DOMAIN"] = api_domain
         os.environ["API_STAGE"] = api_stage
         os.environ["API_PATH"] = api_path
@@ -151,11 +175,20 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         aws_profile = profile if profile else prod_env_settings['environment'][env]['profile']
         domain = prod_env_settings['environment'][env]['domain']
         log_info(logger, aws_profile, domain, env)
-        #### Set API_DOMAIN, API_STAGE, API_PATH as env variable
+        #### Set APP_API_DOMAIN, API_DOMAIN, API_STAGE, API_PATH as env variable
+        
+        if app_api_domain :
+            app_api_domain = app_api_domain
+        elif 'app_api_domain' in prod_env_settings['environment'][env].keys() :
+            app_api_domain =  prod_env_settings['environment'][env]['app_api_domain']
+        else :
+            app_api_domain = ""
+            
         api_domain = api_domain if api_domain else prod_env_settings['environment'][env]['api_domain']
         api_stage = ""
         api_path = api_domain
-        logger.info("Setting up Env variables : API_DOMAIN {}, API_STAGE {}, API_PATH {}".format(api_domain, api_stage, api_path))
+        logger.info("Setting up Env variables : APP_API_DOMAIN {}, API_DOMAIN {}, API_STAGE {}, API_PATH {}".format(app_api_domain, api_domain, api_stage, api_path))
+        os.environ["APP_API_DOMAIN"] = app_api_domain
         os.environ["API_DOMAIN"] = api_domain
         os.environ["API_STAGE"] = api_stage
         os.environ["API_PATH"] = api_path
@@ -177,6 +210,9 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         # STANDALONE: xxx-dev.bobswift.lavadukanam.com
         # PROD: markdown.bobswift.appfire.app
 
+        # default container registry for service modules
+        registry_url = "951171940383.dkr.ecr.us-east-1.amazonaws.com"
+
         app_domain = domain.split(".")[2] + "." + domain.split(".")[3] if env == 'standalone' else "appfire.app"
         app_name = domain.split(".")[0].split("-")[0]
         #brand = domain.split(".")[1] if env != 'standalone' else "bobswift"
@@ -193,16 +229,35 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
         
         #generate_base_url(logger, "web/static/atlassian-connect.json", app_suffix, domain, env)
         #generate_base_url(logger, "atlassian-connect.json", app_suffix, domain, env)
+
         logger.debug("overriding cdk.json with supplied arguments")
         generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand, app_name)
         if stack == 'core':
-            deploy_core(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
+            deploy_core(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
         elif stack == 'biz-service':
             deploy_biz_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
         elif stack == 'app-service':
             deploy_app_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
         elif stack == 'module-service':
             deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
+        elif stack == 'ac-allowlist':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-automation':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-core':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-data-engine-api':
+            deploy_service_module(logger,"ac-data-engine-api",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-datastore':
+            deploy_service_module(logger,"ac-datastore",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-macro-security':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-migrations':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-proxy':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
+        elif stack == 'ac-utils':
+            deploy_service_module(logger,stack,True,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag)
         else:
             shell(logger, "cdk deploy '*' --require-approval "+ require_approval +" --profile " + aws_profile, raise_error=True)
     elif command == 'diff':
@@ -234,12 +289,18 @@ def process(verbose, command, profile, env, stage, stack, app_suffix, build, api
     else:
         shell(logger, "cdk list --profile " + aws_profile, raise_error=True)
 
-
-def deploy_core(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
+def deploy_core(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
     if deploy_biz_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile) :
         if deploy_app_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile) :
-            if deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
-                print('Core services deployed successfully.')
+            if deploy_service_module(logger,"ac-core",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                if deploy_service_module(logger,"ac-allowlist",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                    if deploy_service_module(logger,"ac-automation",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                        if deploy_service_module(logger,"ac-macro-security",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                            if deploy_service_module(logger,"ac-migrations",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                                if deploy_service_module(logger,"ac-proxy",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                                    if deploy_service_module(logger,"ac-utils",False,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+                                        if deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
+                                            print('Core services deployed successfully.')
 
 def deploy_biz_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile):
     if (not os.path.isfile('./node_modules/ac-biz-services/bin/ac-biz-services.js')):
@@ -270,21 +331,54 @@ def deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix
     if (not os.path.isfile('./node_modules/ac-module-services/bin/ac-module-services.js')):
         logger.info('ac-module-services dependency is not installed. Installing latest ac-module-services.')
         shell(logger,"npm install ac-module-services --save-dev",raise_error=True)
-    
-    ## install dependencies for module services
-    logger.info("Installing dependencies for Module Services")
-    shell(logger,"cd ./node_modules/ac-module-services/lib/API/resources/modules-lambda-layer/nodejs && npm install && cd -", raise_error=True)
-    shell(logger,"cd ./node_modules/ac-module-services/lib/jwt/construct/resources/jwt-auth-node-layer/nodejs && npm install && cd -",raise_error=True)
-
-    ## check if automation resources do not exist , then create 
-    if not os.path.isfile('./node_modules/ac-module-services/lib/API/resources/automation-lambda/build/libs/automation-lambda.jar') or not os.path.isfile('./node_modules/ac-module-services/lib/API/resources/automation-lambda/build/distributions/lambda-layer-dependencies.zip'):
-        shell(logger,"cd ./node_modules/ac-module-services/lib/API/resources/automation-lambda && ./gradlew clean build --refresh-dependencies && cd - ",raise_error=True)
 
     ## Deploying ac-module-services
     shell(logger, "cdk deploy '*' --app 'npx ts-node ./node_modules/ac-module-services/bin/ac-module-services.js " + env + " " + app_domain + " " + stage + "'" +
                   " --require-approval never --profile " + aws_profile, raise_error=True)
     return True
 
+def deploy_service_module(logger,service_module,module_services_deploy,app_name,stage,app_domain,brand,env,app_suffix,aws_profile,registry_url,tag):
+    logger.info('Deploying ' + service_module)
+
+    # check to see if a tags yml file exist for the service modules
+    service_module_tags = None
+    try:
+        with open('service-module-tags.yml', 'r') as stream:
+            service_module_tags = yaml.safe_load(stream)
+    except IOError:
+        logger.info("service-module-tags.yml not found!")
+
+    # check if a service module has a tag defined and use that instead of the default (latest)
+    if service_module_tags is not None :
+        if service_module in service_module_tags :
+            if 'tag' in service_module_tags[service_module] :
+                tag = service_module_tags[service_module]['tag']
+
+    # Pull the service_module_image:tag
+    service_module_image = generate_image_url(service_module,registry_url,tag)
+    returnCode = shell(logger,"docker pull " + service_module_image,raise_error=False)
+
+    if returnCode == 1:
+        return False
+
+    cdk_deploy = generate_cdk_deploy(os.environ.get("CDK_DEPLOY_ACCOUNT"), os.environ.get("CDK_DEPLOY_REGION"),aws_profile)
+
+    if service_module == "ac-automation":
+        returnCode = shell(logger,cdk_deploy + " -e STAGE=" + stage  + " " + service_module_image,raise_error=False)
+    elif service_module == "ac-core":
+        returnCode = shell(logger,cdk_deploy + " -e ENV=" + env  + " -e DOMAIN=" + app_domain + " -e APP_SUFFIX=" + app_suffix + " " + service_module_image,raise_error=False)
+    else: 
+        returnCode = shell(logger,cdk_deploy + " -e ENV=" + env + " -e STAGE=" + stage + " " + service_module_image,raise_error=False)
+
+    if returnCode == 1:
+        return False
+
+    if module_services_deploy == True:
+        logger.info('Deploying ac-module-services')
+        deploy_module_services(logger,app_name,stage,app_domain,brand,env,app_suffix,aws_profile)
+
+    return True
+
 def generate_cdk_json(logger, app_suffix, app_domain, env, stack, stage, brand, app_name):
     '''dynamically prepare cdk.json based on command line arguments'''
     
@@ -351,15 +445,26 @@ def shell(logger, cmd, raise_error=False):
     logger.debug("Running : %s" % cmd)
     process = subprocess.run(
         cmd,
-        check=True,
+        check=raise_error,
         shell=True
     )
+
     if raise_error and process.returncode != 0:
         logger.error("Command output:")
         raise ShellCommandFailed("The following command did not succeed: %s" % cmd)
-    
+
     return (process.returncode)
 
+def dts_ecr_login(logger, dts_ecr_profile):
+    logger.info(f'AWS DTS ECR Profile: {dts_ecr_profile}')
+    shell(logger,"aws ecr get-login-password --region us-east-1 --profile " + dts_ecr_profile +
+                 " | docker login --username AWS --password-stdin 951171940383.dkr.ecr.us-east-1.amazonaws.com")
+
+def generate_image_url(service_module,registry_url,tag):
+    return registry_url + "/" + service_module + ":" + tag
+
+def generate_cdk_deploy(cdk_deploy_account, cdk_deploy_region, aws_profile):
+     return f"docker run -v ~/.aws:/appfire/.aws -e CDK_DEPLOY_ACCOUNT={cdk_deploy_account} -e CDK_DEPLOY_REGION={cdk_deploy_region} -e AWS_PROFILE={aws_profile}"
 
 class ShellCommandFailed(Exception):
     """ Executing a shell command failed """
